# compiler for which we are doing this
%{!?compiler: %define compiler intel}
%{!?compilermajor: %define compilermajor 19}
%{!?compilerminor: %define compilerminor 1}
%define compilerversion %{compilermajor}.%{compilerminor}
%define compilerversionnum %{compilermajor}%{compilerminor}
%{!?openmpiversion: %define openmpiversion 4.1.0}
%define openmpinumversion %( printf "%02d%02d%02d" `echo %{openmpiversion} | cut -d. -f1,2,3 --output-delimiter=" "` )

# modules defaults
%define modulefile_path_top /usr/local/share/Modules/modulefiles/fftw

# Turn off hardened stuff
%undefine _hardened_build
%undefine _annotated_build

# now, depending on the compiler we need different things - place them all here for eacy access/changing
#
# GCC Compiler
%if "%{compiler}" == "gcc"
%define compilershort gcc
%define compilerlong gcc
%define compilerversionnum %{nil}
%define compilerruntimesection BuildRequires: gcc-gfortran openmpi-gcc-devel = %{openmpiversion} intel-mpi-modules-devel intel-mpi-modules-gcc
%define compilerdevelsection Requires: gcc-gfortran
%define compilerdocsection #nothing here
%define localdir /usr/local/fftw/gcc
%define modulefile_path %{modulefile_path_top}/gcc
# do nothing special for gcc for build prep
%define compilerbuildprep F77=gfortran FC=gfortran F90=gfortran;. /etc/profile.d/modules.sh
%define compilerload %{nil}
%define cflags %( echo %{optflags} | sed -e 's|-Werror=format-security||' -e 's|-Wp,-D_FORTIFY_SOURCE=2||' -e 's|-fstack-protector-strong||' -e 's|-fstack-clash-protection||' -e 's|-fcf-protection||'  )
%define cxxflags %{cflags}
%define f77flags %{cflags}
%define fcflags  %{cflags}
%define impi 1
%endif
#
# AOCC Compiler
%if "%{compiler}" == "aocc"
%undefine _missing_build_ids_terminate_build
%define compilershort aocc
%define compilerlong aocc
%define compilerversionnum %{nil}
%define compilerruntimesection BuildRequires: aocc-compiler-module openmpi-aocc-devel = %{openmpiversion}
%define compilerdevelsection Requires: aocc-compiler-module
%define compilerdocsection #nothing here
%define localdir /usr/local/fftw/aocc
%define modulefile_path %{modulefile_path_top}/aocc
# do nothing special for gcc for build prep
%define compilerbuildprep export CC=clang CXX=clang++ F77=flang FC=flang F90=flang; module load aocc
%define compilerload %{nil}
%define cflags -O3 -g 
%define cxxflags %{cflags}
%define f77flags %{cflags}
%define fcflags  %{cflags}
%define impi 0
%endif
# Intel compiler
%if "%{compiler}" == "intel"
%define compilershort intel
%define compilerlong intel-%{compilerversion}
%define compilerruntimesection BuildRequires: openmpi-intel%{compilerversionnum}-devel = %{openmpiversion} intel-mpi-modules-devel intel-mpi-modules-intel \
Provides: fftw2-intel = %{version}-%{release}
%define compilerdevelsection Provides: fftw2-intel-devel = %{version}-%{release}
%define compilerdocsection Provides: fftw2-intel-doc = %{version}-%{release}
%define localdir /usr/local/fftw/intel-%{compilerversion}
%define modulefile_path %{modulefile_path_top}/intel-%{compilerversion}
# for intel we need to just specify our compilers
%define compilerbuildprep export LDFLAGS="-Wl,--build-id" CC=icc CXX=icpc F77=ifort FC=ifort F90=ifort; . /etc/profile.d/modules.sh
%define compilerload module load intel
%define cflags -O3 -g -pipe -Wall
%define cxxflags %{cflags}
%define f77flags %{cflags}
%define fcflags  %{cflags}
%define impi 1
%endif
#
# PGI compiler
%if "%{compiler}" == "pgi" || "%{compiler}" == "nvhpc"
%define compilershort %{compiler}
%define compilerruntimesection BuildRequires: openmpi-%{compilershort}%{compilerversionnum}-devel = %{openmpiversion} \
Provides: fftw2-%{compilershort} = %{version}-%{release}
%define compilerdevelsection Provides: fftw2-%{compilershort}-devel = %{version}-%{release}
%define compilerdocsection Provides: fftw2-%{compilershort}-doc = %{version}-%{release}
%define localdir /usr/local/%{compilershort}
%define modulefile_path %{modulefile_path_top}/intel-%{compilerversion}
# for pgi we need to just specify our compilers
# we also need to force -fPIC flag to get shared libraries and other things
%define compilerbuildprep export LDFLAGS="-Wl,--build-id"
%define compilerload module load nvhpc
%define cflags -O3 -g
%define cxxflags %{cflags}
%define f77flags %{cflags}
%define fcflags  %{cflags}
#define ldflags  -lpthread
%define impi 0
%endif
#
# PGI compiler
%if "%{compiler}" == "pathscale"
%define compilershort pathscale
%define compilerruntimesection BuildRequires: openmpi-pathscale-devel
Provides: fftw2-pathscale = %{version}-070.%{release}
%define compilerdevelsection Requires: pathscale-workstation70\
Provides: fftw2-pathscale-devel = %{version}-070.%{release}
%define compilerdocsection Provides: fftw2-pathscale-doc = %{version}-070.%{release}
%define localdir /usr/local/pathscale
# for pathscale we need to just specify our compilers
# we also need to force -fPIC flag to get shared libraries and other things
%define compilerbuildprep export LDFLAGS="-Wl,--build-id" CC=pathcc CXX=pathCC F77=pathf90 FC=pathf90 F90=pathf90 lt_cv_prog_cc_pic=-fPIC lt_cv_prog_cc_wl='-Wl,' lt_cv_prog_cc_static='-static'; . /etc/profile.d/modules.sh 
%define cflags -O3
%define cxxflags %{cflags}
%define f77flags %{cflags}
%define fcflags  %{cflags}
#define ldflags  -lpthread
%endif

Name:           fftw2-%{compiler}%{compilerversionnum}
Version:        2.1.5
Release:        29%{?dist}
Summary:        Fast Fourier Transform library
%define 	real_name fftw

Group:          System Environment/Libraries
License:        GPL
URL:            http://www.fftw.org/
Source0:        ftp://ftp.fftw.org/pub/fftw/fftw-%{version}.tar.gz
Patch0:         rfftw_f77_mpi.c.patch
BuildRoot:      %{_tmppath}/%{name}-%{version}-%{release}-root-%(%{__id_u} -n)
BuildRequires:  rsync
Requires:	environment-modules-local
BuildRequires:	environment-modules-local
%{compilerruntimesection}

%description
FFTW is a C subroutine library for computing the Discrete Fourier Transform
(DFT) in one or more dimensions, of both real and complex data, and of
arbitrary input size. We believe that FFTW, which is free software, should
become the FFT library of choice for most applications. Our benchmarks,
performed on on a variety of platforms, show that FFTW's performance is
typically superior to that of other publicly available FFT software.

This rpm was compiled with %{compiler} family of compilers.

%package        devel
Summary:        Headers, libraries and docs for the FFTW library
Group:          Development/Libraries
Requires:       %{name} = %{version}-%{release}

%description    devel
FFTW is a C subroutine library for computing the Discrete Fourier Transform
(DFT) in one or more dimensions, of both real and complex data, and of
arbitrary input size. We believe that FFTW, which is free software, should
become the FFT library of choice for most applications. Our benchmarks,
performed on on a variety of platforms, show that FFTW's performance is
typically superior to that of other publicly available FFT software.

This package contains header files and development libraries needed to
develop programs using the FFTW fast Fourier transform library.

This rpm was compiled with %{compiler} family of compilers.

%package        openmpi%{openmpinumversion}
Summary:        Libraries for the FFTW library for openmpi %{openmpiversion}
Group:          Development/Libraries
#Requires:       %{name} = %{version}-%{release}

%description    openmpi%{openmpinumversion}
FFTW is a C subroutine library for computing the Discrete Fourier Transform
(DFT) in one or more dimensions, of both real and complex data, and of
arbitrary input size. We believe that FFTW, which is free software, should
become the FFT library of choice for most applications. Our benchmarks,
performed on on a variety of platforms, show that FFTW's performance is
typically superior to that of other publicly available FFT software.

This package contains header files and development libraries needed to
develop programs using the FFTW fast Fourier transform library.

This rpm was compiled with %{compiler} family of compilers and contains
mpi related material compiled with openmpi %{openmpiversion}.

%package        openmpi%{openmpinumversion}-devel
Summary:        Development Libraries and docs for the FFTW library for openmpi %{openmpiversion}
Group:          Development/Libraries
Requires:       %{name}-openmpi%{openmpinumversion} = %{version}-%{release}

%description    openmpi%{openmpinumversion}-devel
FFTW is a C subroutine library for computing the Discrete Fourier Transform
(DFT) in one or more dimensions, of both real and complex data, and of
arbitrary input size. We believe that FFTW, which is free software, should
become the FFT library of choice for most applications. Our benchmarks,
performed on on a variety of platforms, show that FFTW's performance is
typically superior to that of other publicly available FFT software.

This package contains header files and development libraries needed to
develop programs using the FFTW fast Fourier transform library.

This rpm was compiled with %{compiler} family of compilers and contains
mpi related material compiled with openmpi necessary for development work.

%if %{impi}
%package        impi
Summary:        Libraries for the FFTW library for intel mpi
Group:          Development/Libraries
#Requires:       %{name} = %{version}-%{release}

%description    impi
FFTW is a C subroutine library for computing the Discrete Fourier Transform
(DFT) in one or more dimensions, of both real and complex data, and of
arbitrary input size. We believe that FFTW, which is free software, should
become the FFT library of choice for most applications. Our benchmarks,
performed on on a variety of platforms, show that FFTW's performance is
typically superior to that of other publicly available FFT software.

This package contains header files and development libraries needed to
develop programs using the FFTW fast Fourier transform library.

This rpm was compiled with %{compiler} family of compilers and contains
mpi related material compiled with intel mpi

%package        impi-devel
Summary:        Development Libraries and docs for the FFTW library for intel mpi
Group:          Development/Libraries
Requires:       %{name}-impi = %{version}-%{release}

%description    impi-devel
FFTW is a C subroutine library for computing the Discrete Fourier Transform
(DFT) in one or more dimensions, of both real and complex data, and of
arbitrary input size. We believe that FFTW, which is free software, should
become the FFT library of choice for most applications. Our benchmarks,
performed on on a variety of platforms, show that FFTW's performance is
typically superior to that of other publicly available FFT software.

This package contains header files and development libraries needed to
develop programs using the FFTW fast Fourier transform library.

This rpm was compiled with %{compiler} family of compilers and contains
mpi related material compiled with intel mpi necessary for development work.
%endif

%prep
%setup -q -c %{real_name}-%{version}
mv %{real_name}-%{version} single
cd single
%patch0 -p0
cd ..
cp -a single double

%build
build_one() {
D=%{localdir}/$1/%{version}
mkdir $1
rsync -arH single double $1/
pushd $1
pushd double
	%ifarch i386
		%configure \
			--prefix=$D \
			--libdir=$D \
			--includedir=$D \
			--infodir=$D/share/info \
			--enable-shared \
			--enable-threads \
			--enable-mpi \
			--enable-i386-hacks
	%else
		%configure \
			--prefix=$D \
			--libdir=$D/%{_lib} \
			--includedir=$D/include \
			--infodir=$D/share/info \
			--enable-shared \
			--enable-mpi \
			--enable-threads
	%endif
	make %{?_smp_mflags}
popd
pushd single
	%configure \
		--prefix=$D \
		--libdir=$D/%{_lib} \
		--includedir=$D/include \
		--infodir=$D/share/info \
		--enable-shared \
		--enable-type-prefix \
		--enable-threads \
		--enable-mpi \
		--enable-float
	make %{?_smp_mflags}
popd
popd
module purge
}

CFLAGS="%{?cflags:%{cflags}}%{!?cflags:$RPM_OPT_FLAGS}"
CXXFLAGS="%{?cxxflags:%{cxxflags}}%{!?cxxflags:$RPM_OPT_FLAGS}"
F77FLAGS="%{?f77flags:%{f77flags}}%{!?f77flags:$RPM_OPT_FLAGS}"
FFLAGS="%{?f77flags:%{f77flags}}%{!?f77flags:$RPM_OPT_FLAGS}"
FCFLAGS="%{?fcflags:%{fcflags}}%{!?fcflags:$RPM_OPT_FLAGS}"
LDFLAGS="%{?ldflags:%{ldflags}}"

export CFLAGS CXXFLAGS F77FLAGS FCFLAGS FFLAGS LDFLAGS

%{compilerbuildprep}
%{compilerload}
module load openmpi
build_one openmpi-%{openmpiversion}

%if %{impi}
%{compilerload}
module load intel-mpi/%{compiler}
build_one intel-mpi
%endif

%install
rm -rf ${RPM_BUILD_ROOT}

# First param is the directory name and also module name
# Second param is the module requirement
install_one() {
D=%{localdir}/$1/%{version}
pushd $1
pushd double
	make install DESTDIR=${RPM_BUILD_ROOT}
popd
pushd single
	make install DESTDIR=${RPM_BUILD_ROOT}
popd
module purge
popd
mkdir -p ${RPM_BUILD_ROOT}/%{modulefile_path}/$1
cat > ${RPM_BUILD_ROOT}/%{modulefile_path}/$1/%{version} <<ENDMODULE
#%Module

# NOTE: This is an automatically-generated file!  (generated by the
# fftw rpm).  Any changes made here will be lost a) if the RPM is
# uninstalled, or b) if the RPM is upgraded or uninstalled.

proc ModulesHelp { } {
   puts stderr "This module adds fftw %{version} for %{compilerlong} to various paths"
}

module-whatis   "Sets up fftw %{version} for %{compilerlong} in your environment"

prepend-path LD_LIBRARY_PATH "$D/%{_lib}"
prepend-path LIBRARY_PATH "$D/%{_lib}"
setenv FFTW_DIR "%{localdir}"
append-path -d { } LDFLAGS "-L$D/%{_lib}"
append-path -d { } INCLUDE "-I$D/include"
append-path CPATH "$D/include"
append-path -d { } FFLAGS "-I$D/include"
append-path -d { } LOCAL_LDFLAGS "-L$D/%{_lib}"
append-path -d { } LOCAL_INCLUDE "-I$D/include"
append-path -d { } LOCAL_CFLAGS "-I$D/include"
append-path -d { } LOCAL_FFLAGS "-I$D/include"
append-path -d { } LOCAL_CXXFLAGS "-I$D/include"
conflict fftw
prereq $2
ENDMODULE
}

%{compilerbuildprep}

%{compilerload}
module load openmpi
install_one openmpi-%{openmpiversion} openmpi

%if %{impi}
%{compilerload}
module load intel-mpi/%{compiler}
install_one intel-mpi intel-mpi
%endif

rm -f doc/Makefile*
find ${RPM_BUILD_ROOT} -type f -name "*.la" -exec rm -f {} ';'

## move info files under %{localdir}
#mkdir -p $RPM_BUILD_ROOT/%{localdir}/share/info
#mv  $RPM_BUILD_ROOT/%{_prefix}/share/info/fftw*info* $RPM_BUILD_ROOT/%{localdir}/share/info/

%clean
rm -rf ${RPM_BUILD_ROOT}

%files openmpi%{openmpinumversion}
%defattr(-,root,root,-)
%doc single/AUTHORS single/COPYRIGHT
%dir %{localdir}
%dir %{localdir}/openmpi-%{openmpiversion}
%dir %{localdir}/openmpi-%{openmpiversion}/%{version}
%dir %{localdir}/openmpi-%{openmpiversion}/%{version}/%{_lib}
%{localdir}/openmpi-%{openmpiversion}/%{version}/%{_lib}/*.so.*
%dir %{modulefile_path_top}
%dir %{modulefile_path}
%{modulefile_path}/openmpi-%{openmpiversion}

%files openmpi%{openmpinumversion}-devel
%defattr(-,root,root,-)
%{localdir}/openmpi-%{openmpiversion}/%{version}/include
%{localdir}/openmpi-%{openmpiversion}/%{version}/share
%{localdir}/openmpi-%{openmpiversion}/%{version}/%{_lib}/*.a
%{localdir}/openmpi-%{openmpiversion}/%{version}/%{_lib}/*.so

%if %{impi}
%files impi
%defattr(-,root,root,-)
%doc single/AUTHORS single/COPYRIGHT
%dir %{localdir}/intel-mpi
%dir %{localdir}/intel-mpi/%{version}
%dir %{localdir}/intel-mpi/%{version}/%{_lib}
%{localdir}/intel-mpi/%{version}/%{_lib}/*.so.*
%dir %{modulefile_path_top}
%dir %{modulefile_path}
%{modulefile_path}/intel-mpi

%files impi-devel
%defattr(-,root,root,-)
%{localdir}/intel-mpi/%{version}/include
%{localdir}/intel-mpi/%{version}/share
%{localdir}/intel-mpi/%{version}/%{_lib}/*.a
%{localdir}/intel-mpi/%{version}/%{_lib}/*.so
%endif

%changelog
* Sun Apr 08 2012 Josko Plazonic <plazonic@math.princeton.edu>
- add support for intel mpi
 
* Mon Mar 12 2012 Josko Plazonic <plazonic@math.princeton.edu>
- add support for SGI's mpt

* Mon Mar 17 2008 Josko Plazonic <plazonic@math.princeton.edu>
- adapt to artemis, i.e. suse

* Tue Aug 29 2006 José Matos <jamatos[AT]fc.up.pt> - 2.1.5-13
- Rebuild for FE6

* Sat Feb 18 2006 José Matos <jamatos[AT]fc.up.pt> - 2.1.5-12
- Rebuild for FC-5.

* Wed Feb  1 2006 José Matos <jamatos[AT]fc.up.pt> - 2.1.5-11
- Fix incomplete substitution

* Wed Feb  1 2006 José Matos <jamatos[AT]fc.up.pt> - 2.1.5-10
- Add disttag to release.

* Wed Feb  1 2006 José Matos <jamatos[AT]fc.up.pt> - 2.1.5-9
- Rename package to fftw2.

* Mon May 23 2005 Michael Schwendt <mschwendt[AT]users.sf.net> - 2.1.5-8
- BuildReq gcc-gfortran (#156490).

* Sun May 22 2005 Jeremy Katz <katzj@redhat.com> - 2.1.5-7
- rebuild on all arches
- buildrequire compat-gcc-32-g77

* Fri Apr  7 2005 Michael Schwendt <mschwendt[AT]users.sf.net>
- rebuilt

* Wed Nov 10 2004 Matthias Saou <http://freshrpms.net/> 2.1.5-5
- Bump release to provide Extras upgrade path.

* Tue Apr 06 2004 Phillip Compton <pcompton[AT]proteinmedia.com> - 0:2.1.5-0.fdr.4
- BuildReq gcc-g77.

* Mon Sep 22 2003 Phillip Compton <pcompton[AT]proteinmedia.com> - 0:2.1.5-0.fdr.3
- Dropped post/preun scripts for info.

* Wed Sep 17 2003 Phillip Compton <pcompton[AT]proteinmedia.com> - 0:2.1.5-0.fdr.2
- Remove aesthetic comments.
- buildroot -> RPM_BUILD_ROOT.
- post/preun for info files.

* Mon Apr 07 2003 Phillip Compton <pcompton[AT]proteinmedia.com> - 0:2.1.5-0.fdr.1
- Updated to 2.1.5.

* Tue Apr 01 2003 Phillip Compton <pcompton[AT]proteinmedia.com> - 0:2.1.4-0.fdr.2
- Added Epoch:0.
- Added ldconfig to post and postun.

* Sun Mar 22 2003 Phillip Compton <pcompton[AT]proteinmedia.com> - 2.1.4-0.fdr.1
- Updated to 2.1.4.

* Fri Mar 14 2003 Phillip Compton <pcompton[AT]proteinmedia.com> - 2.1.3-0.fdr.1
- Fedorafied.

* Mon Oct 21 2002 Matthias Saou <matthias.saou@est.une.marmotte.net>
- Initial RPM release.

